/***************************************************************************
 * 
 * Copyright (c) 2014 Baidu.com, Inc. All Rights Reserved
 * $Id$ 
 * 
 **************************************************************************/
 
 
 
/**
 * src/p2.js ~ 2014/04/16 15:09:32
 * @author leeight(liyubei@baidu.com)
 * @version $Revision$ 
 * @description 
 * 把如下的地址从registry.npmjs.org下载下来，然后保存到本地
 * http://npm.internal.baidu.com/-/shasum=d84d2dd0a7d52fffa9340778925321850825ab68/baidu/-/baidu-0.0.1.tgz
 **/

var http = require( 'http' );
var proxy = require( 'http-proxy' ).createProxyServer({});

proxy.on('error', function (err, req, res) {
    res.writeHead(500, {
        'Content-Type': 'text/plain'
    });

    res.end('Something went wrong. And we are reporting a custom error message.');
});

var server = require('http').createServer(function(req, res) {
    var pattern = /^\/\-\/shasum=([^\/]+)(\/.*)/g;
    var match = pattern.exec( req.url );
    if ( !match ) {
        res.end( 'Invalid request url format' );
        return;
    }

    var shasum = match[ 1 ];
    var url = match[ 2 ];

    if ( exists( url ) ) {
        res.writeHead( 200, {
            'Content-Type': 'application/octet-stream',
            'Expires': 'Wed, 16 Apr 2024 13:32:04 GMT',
            'Connection': 'Close',
            'Content-Length': size( resolve( url ) )
        });
        res.end( require( 'fs' ).readFileSync( resolve( url ) ) );
        return;
    }

    req.headers.host = 'registry.npmjs.org';
    req.url = url;

    var end = res.end;
    var write = res.write;

    var buffer = [];
    res.write = function( data ) {
        buffer.push( data );
    }
    res.end = function( data, encoding) {
        if ( data ) {
            buffer.push( data );
        }

        var s = require( 'crypto' ).createHash( 'sha1' );
        var b = Buffer.concat( buffer );
        s.update( b );
        if ( s.digest( 'hex' ) === shasum ) {
            save( b, url, shasum );
        }

        res.write = write;
        end.call( res, b, encoding );
    }

    proxy.proxyRequest(req, res, { target: 'http://registry.npmjs.org' } );
});

/**
 * @param {string} url 请求的url.
 * @return {string}
 */
function resolve( url ) {
    var path = require( 'path' );
    var fullpath = path.join( __dirname, '..', 'data', url );

    return path.normalize( fullpath );
};

/**
 * 获取文件的大小.
 * @param {string} path 文件的路径.
 * @return {number}
 */
function size( path ) {
    var fs = require( 'fs' );
    return fs.statSync( path ).size;
}

/**
 * 检查本地是否有缓存的内容.
 * @param {string} url 请求的url.
 *
 * @return {boolean}
 */
function exists( url ) {
    var fs = require( 'fs' );
    return fs.existsSync( resolve( url ) );
}

/**
 * @param {Buffer} buffer 数据.
 * @param {string} url 请求的路径.
 * @param {string} shasum 校验码.
 *
 * @return {boolean}
 */
function save( buffer, url, shasum ) {
    // url => /baidu/-/baidu-0.0.1.tar.gz
    var path = require( 'path' );
    var fs = require( 'fs' );

    var fullpath = resolve( url );
    var dir = path.dirname( fullpath );
    var file = path.basename( fullpath );

    if ( !fs.existsSync( dir ) ) {
        require( 'mkdirp' ).sync( dir );
    }

    fs.writeFileSync( fullpath, buffer );

    return true;
}

server.listen( 8986 );





















/* vim: set ts=4 sw=4 sts=4 tw=100: */
